#!/usr/bin/env python
# coding=utf-8
"""
@athor:weifeng.guo
@data:2019/3/26 13:56
@filename:flask_reset_client
"""
# pylint: disable=wrong-import-position, relative-import
import sys
import os
import re
import argparse
import platform
sys.path.append(os.path.abspath(os.path.join(os.getcwd(), "..", "..", "..")))
from rest_client import RestClient


def get_function_list(file_path):
    func_list = list()
    class_name = None
    for line in open(file_path).readlines():
        if r"class Test" in line:
            class_name = re.findall("class ([a-zA-Z_]+)", line)[0]

        if r"    def test_" in line and class_name:
            test_name = re.findall("def ([a-zA-Z0-9_]+)", line)[0]
            func_list.append("{}:{}.{}".format(file_path, class_name, test_name))

    return func_list


def list_test_case(file_list, keyword=None):
    for file_path in file_list:
        if keyword:
            if keyword in file_path:
                if platform.system() == "Linux":
                    print(file_path.replace(keyword, "\033[1;31m{}\033[0m".format(keyword)))
                else:
                    print(file_path)

            for func_name in get_function_list(file_path):
                if keyword in func_name:
                    if platform.system() == "Linux":
                        print(func_name.replace(keyword, "\033[1;31m{}\033[0m".format(keyword)))
                    else:
                        print(func_name)
        else:
            print(file_path)


def run(args):
    rest_client = RestClient(args.targetip)
    if args.list:
        if args.cmd == "all":
            result_list = rest_client.test_case.list(args.cmd)
            list_test_case(result_list)
        else:
            result_list = rest_client.test_case.list(args.cmd)
            list_test_case(result_list)
    elif args.nose:
        print(rest_client.test_case.run(None, ip=args.targetip, command=args.cmd,
                                        loglevel=args.loglevel, controller=args.controller,
                                        flash=args.flashtype, extra=args.extra,
                                        nose=args.nose)['msg'])
    elif args.regress:
        print(rest_client.test_case.run(None, ip=args.targetip, command=args.cmd,
                                        loglevel=args.loglevel, controller=args.controller,
                                        flash=args.flashtype, extra=args.extra,
                                        regress=args.regress)['msg'])
    else:
        print(rest_client.test_case.run(None, ip=args.targetip, command=args.cmd,
                                        loglevel=args.loglevel, controller=args.controller,
                                        flash=args.flashtype, extra=args.extra,
                                        python=args.python)['msg'])


if __name__ == '__main__':
    PARSER = argparse.ArgumentParser(description='Restful Client')
    PARSER.add_argument('-t', '--targetip', required=True, help='Target IP')
    PARSER.add_argument('-c', '--controller', default='TAHOE', help='Controller of CNEX SSD')
    PARSER.add_argument('-f', '--flashtype', default='BICS', help='Flash type of CNEX SSD')
    PARSER.add_argument('-L', '--loglevel', default='DEBUG',
                        choices=('DEBUG', 'INFO', 'WARN', 'ERROR'), help='Log level')
    GROUP = PARSER.add_mutually_exclusive_group(required=True)
    GROUP.add_argument('-l', '--list', action='store_true', help='List test cases')
    GROUP.add_argument('-n', '--nose', action='store_true', help='Run nosetests command')
    GROUP.add_argument('-p', '--python', action='store_true', help='Run python command')
    GROUP.add_argument('-r', '--regress', action='store_true', help='Regress nosetests file')
    PARSER.add_argument('cmd', help='Command Line')
    PARSER.add_argument('extra', nargs=argparse.REMAINDER)
    ARGS = PARSER.parse_args()

    sys.exit(run(ARGS))



# coding=utf-8
# pylint: disable=wrong-import-position, relative-import
import sys
import os
import requests
sys.path.append(os.path.join(os.path.dirname(__file__)))
from resources.testsuite_resource import TestSuiteResource
from resources.testcase_resource import TestCaseResource
from resources.operation_resource import OperationResource
from resources.state_resource import StateResource


class RestClient(object):

    def __init__(self, host, port=5000, time_out=5):
        __session = requests.Session()
        self.test_suite = TestSuiteResource(host, port, __session, time_out=time_out)
        self.test_case = TestCaseResource(host, port, __session, time_out=time_out)
        self.operation = OperationResource(host, port, __session, time_out=time_out)
        self.state = StateResource(host, port, __session, time_out=time_out)

if __name__ == '__main__':
    RC = RestClient("172.29.130.138")
    print(RC.test_suite.get_test_lists("fio"))
    # print(RC.test_case.list("fio"))
    # print(RC.test_case.run("test_fio_linux:TestFioLinux.test_rand_read", "async"))
    # print(RC.test_case.get_async_result("sjalvKeaDY"))
    # print(RC.test_case.stop_tests())



# coding=utf-8
# pylint: disable=import-error, broad-except
import json
from resources.models.helper import rest_get_call
from resources.models.helper import rest_post_json_call
from resources.models.helper import rest_delete_call
from resources.models.helper import RestError


class TestCaseResource(object):

    def __init__(self, host, port, session, time_out):
        self.host = host
        self.port = port
        self.session = session
        self.time_out = time_out

    def list(self, filter_):
        """
        :param filter_: list filter
        :return: type: list, return test case list meet the filter
        """
        url_ = "http://{0}:{1}/tc/{2}".format(self.host, self.port, filter_)
        result = rest_get_call(url_, self.session, self.time_out)
        return result["resource"]

    def run(self, test_case, mode="sync", **kwargs):
        """
        :param test_case: type string, test case name
        :param  mode: sync or async, run test case
        :return: type bool, return test case run result
        """
        data = {"tc": test_case, "mode":mode}
        for key in kwargs:
            data[key] = kwargs[key]
        try:
            url_ = "http://{0}:{1}/tc".format(self.host, self.port)
            result = rest_post_json_call(url_, self.session, json.dumps(data), self.time_out)
        except RestError:
            pass
        return result["resource"]

    def get_async_result(self, key):
        url_ = "http://{0}:{1}/tc/results/{2}".format(self.host, self.port, key)
        try:
            result = rest_get_call(url_, self.session, self.time_out)
        except RestError:
            pass
        return result["resource"]

    def stop_tests(self):
        url_ = "http://{0}:{1}/tc".format(self.host, self.port)
        try:
            result = rest_delete_call(url_, self.session, self.time_out)
        except RestError:
            pass
        return result["resource"]



# coding=utf-8
# pylint: disable=import-error
import json
from resources.models.helper import rest_post_json_call


class OperationResource(object):

    def __init__(self, host, port, session, time_out):
        self.host = host
        self.port = port
        self.session = session
        self.time_out = time_out

    def upgrade(self, fw_path, device_index=1, slot=2):
        data = {"operate_name":"upgrade", "fw": fw_path, "device_index":device_index, "slot":slot}
        url_ = "http://{0}:{1}/operation".format(self.host, self.port)
        result = rest_post_json_call(url_, self.session, json.dumps(data), self.time_out)
        return result["resource"]
